@@ -15,13 +15,17 @@ module Releases
       end
 
       def name
-        params[:name]
+        params[:name] || tag_name
       end
 
       def description
         params[:description]
       end
 
+      def released_at
+        params[:released_at]
+      end
+
       def release
         strong_memoize(:release) do
           project.releases.find_by_tag(tag_name)
